#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sqlite3.h>

#include "bookmarks-scope.h"
#include "shared-values.h"

static void bookmarks_scope_class_init (BookmarksScopeClass *klass);
static void bookmarks_scope_init       (BookmarksScope *self);
static void bookmarks_scope_dispose    (GObject *object);
static void bookmarks_scope_finalize   (GObject *object);

static void active_search_changed (UnityScope * scope, UnityLensSearch * search, UnitySearchType search_type, GCancellable * cancel, gpointer user_data);
static void search_core (UnityScope * scope, UnityLensSearch * search, DeeModel * results, BookmarksScope * bookmarks);

G_DEFINE_TYPE (BookmarksScope, bookmarks_scope, G_TYPE_OBJECT);

static void
bookmarks_scope_class_init (BookmarksScopeClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	object_class->dispose = bookmarks_scope_dispose;
	object_class->finalize = bookmarks_scope_finalize;

	return;
}

static void
bookmarks_scope_init (BookmarksScope *self)
{
	self->scope = NULL;
	self->bookmark_db = NULL;

	self->scope = unity_scope_new(SCOPE_PATH);

	g_signal_connect(G_OBJECT(self->scope), "search-changed", G_CALLBACK(active_search_changed), self);

	gchar * basepath = g_build_filename(g_get_home_dir(), ".mozilla", "firefox", NULL);
	gboolean gotbookmarks = FALSE;
	if (g_file_test(basepath, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
		GDir * ffdir = g_dir_open(basepath, 0, NULL);
		if (ffdir != NULL) {
			const gchar * direntry = NULL;
			while ((direntry = g_dir_read_name(ffdir)) != NULL && !gotbookmarks) {
				if (g_str_has_suffix(direntry, ".default")) {
					gchar * dbname = g_build_filename(basepath, direntry, "places.sqlite", NULL);
					if (g_file_test(dbname, G_FILE_TEST_EXISTS)) {
						if (sqlite3_open(dbname, &(self->bookmark_db)) != SQLITE_OK) {
							self->bookmark_db = NULL;
							g_warning("Unable to open bookmarks DB: %s", dbname);
						} else {
							gotbookmarks = TRUE;
							g_debug("Got bookmarks: %s", dbname);
						}
					}
					g_free(dbname);
				}
			}
			g_dir_close(ffdir);
		}
	}
	g_free(basepath);

	if (!gotbookmarks) {
		g_warning("Unable to find bookmarks");
	}

	return;
}

static void
bookmarks_scope_dispose (GObject *object)
{
	BookmarksScope * self = BOOKMARKS_SCOPE(object);

	if (self->scope != NULL) {
		g_object_unref(G_OBJECT(self->scope));
		self->scope = NULL;
	}

	if (self->bookmark_db != NULL) {
		sqlite3_close(self->bookmark_db);
		self->bookmark_db = NULL;
	}

	G_OBJECT_CLASS (bookmarks_scope_parent_class)->dispose (object);
	return;
}

static void
bookmarks_scope_finalize (GObject *object)
{

	G_OBJECT_CLASS (bookmarks_scope_parent_class)->finalize (object);
	return;
}

BookmarksScope *
bookmarks_scope_new (void)
{
	return BOOKMARKS_SCOPE(g_object_new(BOOKMARKS_SCOPE_TYPE, NULL));
}

UnityLensSearch *
unity_scope_get_active_search(UnityScope * scope)
{
	UnityLensSearch * retval = NULL;

	g_object_get(G_OBJECT(scope),
	             "active-search", &retval,
	             NULL);

	return retval;
}

UnityLensSearch *
unity_scope_get_active_global_search(UnityScope * scope)
{
	UnityLensSearch * retval = NULL;

	g_object_get(G_OBJECT(scope),
	             "active-global-search", &retval,
	             NULL);

	return retval;
}

static void
active_search_changed (UnityScope * scope, UnityLensSearch * search, UnitySearchType search_type, GCancellable * cancel, gpointer user_data)
{
	g_debug("Search Changed");

	g_return_if_fail(scope != NULL);

	/* Get the results and make sure they're cleared */
	DeeModel * results = DEE_MODEL(unity_scope_get_results_model(scope));
	g_return_if_fail(results != NULL);
	dee_model_clear(results);

	/* Grab the search */
	//UnityLensSearch * search = unity_scope_get_active_search(scope);
	g_return_if_fail(search != NULL);

	/* Get our base object out */
	BookmarksScope * bookmarks = BOOKMARKS_SCOPE(user_data);
	g_return_if_fail(bookmarks != NULL);

	search_core(scope, search, results, bookmarks);
	return;
}

static void
search_core (UnityScope * scope, UnityLensSearch * search, DeeModel * results, BookmarksScope * bookmarks)
{
	/* Grab the actual search string */
	const gchar * search_string = unity_lens_search_get_search_string(search);
	g_return_if_fail(search_string != NULL);

	/* Break it up into a set of tokens */
	gchar ** tokens = g_strsplit(search_string, " ", 100);
	gchar * sql = NULL;

	/* If we've got tokens we need to put them into the
	   SQL query.  Else, we don't need to restrict it. */
	if (tokens[0] != NULL) {
		gchar * sql_likes = g_strjoinv("%%'and moz_bookmarks.title like '%%", tokens);
		sql = g_strdup_printf("select moz_bookmarks.title, moz_places.url from moz_bookmarks, moz_places where moz_places.id = moz_bookmarks.fk and moz_bookmarks.title is not null and moz_bookmarks.type = 1 and moz_bookmarks.title like '%%%s%%'  order by moz_bookmarks.lastModified;", sql_likes);
		g_free(sql_likes);
	} else {
		sql = g_strdup("select moz_bookmarks.title, moz_places.url from moz_bookmarks, moz_places where moz_places.id = moz_bookmarks.fk and moz_bookmarks.title is not null and moz_bookmarks.type = 1 order by moz_bookmarks.lastModified;");
	}

	g_strfreev(tokens);

	sqlite3_stmt * stmt = NULL;
	int error_code = sqlite3_prepare_v2(bookmarks->bookmark_db, sql, -1, &stmt, NULL);
	if (error_code == SQLITE_OK) {
		while (sqlite3_step(stmt) == SQLITE_ROW) {
			const guchar * title = sqlite3_column_text(stmt, 0);
			const guchar * url = sqlite3_column_text(stmt, 1);

			g_debug("Sending: %s", title);

			dee_model_append(results,
			   /* URI */       url,
			   /* icon */      "firefox-panel",
			   /* category */  0,
			   /* mimetype */  "",
			   /* name */      title,
			   /* comment */   NULL,
			   /* DND URI */   url
			);
		} // while

		sqlite3_finalize(stmt);
	} else {
		g_warning("Unable to build SQL statement: %s", sql);
		g_warning("Error (%d): %s", error_code, sqlite3_errmsg(bookmarks->bookmark_db));
	} // if prepare

	g_free(sql);

	return;
}
